@@ -1,36 +1,24 @@
 /*
- *   Copyright (C) 2014 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2014 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library/Lesser General Public License
- *   version 2, or (at your option) any later version, as published by the
- *   Free Software Foundation
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library/Lesser General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *   SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "PaginateModel.h"
-#include <QtMath>
 #include "discover_debug.h"
+#include <QtMath>
 
 class PaginateModel::PaginateModelPrivate
 {
 public:
     int m_firstItem = 0;
     int m_pageSize = 0;
-    QAbstractItemModel* m_sourceModel = nullptr;
+    QAbstractItemModel *m_sourceModel = nullptr;
     bool m_hasStaticRowCount = false;
+    bool m_modelResetInProgress = false;
 };
 
-PaginateModel::PaginateModel(QObject* object)
+PaginateModel::PaginateModel(QObject *object)
     : QAbstractListModel(object)
     , d(new PaginateModelPrivate)
 {
@@ -47,10 +35,12 @@ void PaginateModel::setFirstItem(int row)
 {
     Q_ASSERT(row >= 0 && row < d->m_sourceModel->rowCount());
     if (row != d->m_firstItem) {
+        d->m_modelResetInProgress = true;
         beginResetModel();
         d->m_firstItem = row;
         endResetModel();
-        emit firstItemChanged();
+        d->m_modelResetInProgress = false;
+        Q_EMIT firstItemChanged();
     }
 }
 
@@ -65,9 +55,9 @@ void PaginateModel::setPageSize(int count)
         const int oldSize = rowsByPageSize(d->m_pageSize);
         const int newSize = rowsByPageSize(count);
         const int difference = newSize - oldSize;
-        if (difference==0) {
+        if (difference == 0) {
             d->m_pageSize = count;
-        } else if (difference>0) {
+        } else if (difference > 0) {
             beginInsertRows(QModelIndex(), d->m_pageSize, d->m_pageSize + difference - 1);
             d->m_pageSize = count;
             endInsertRows();
@@ -76,22 +66,23 @@ void PaginateModel::setPageSize(int count)
             d->m_pageSize = count;
             endRemoveRows();
         }
-        emit pageSizeChanged();
+        Q_EMIT pageSizeChanged();
     }
 }
 
-QAbstractItemModel* PaginateModel::sourceModel() const
+QAbstractItemModel *PaginateModel::sourceModel() const
 {
     return d->m_sourceModel;
 }
 
-void PaginateModel::setSourceModel(QAbstractItemModel* model)
+void PaginateModel::setSourceModel(QAbstractItemModel *model)
 {
     if (d->m_sourceModel) {
         disconnect(d->m_sourceModel, nullptr, this, nullptr);
     }
 
     if (model != d->m_sourceModel) {
+        d->m_modelResetInProgress = true;
         beginResetModel();
         d->m_sourceModel = model;
         if (model) {
@@ -120,35 +111,34 @@ void PaginateModel::setSourceModel(QAbstractItemModel* model)
             connect(d->m_sourceModel, &QAbstractItemModel::modelReset, this, &PaginateModel::pageCountChanged);
         }
         endResetModel();
-        emit sourceModelChanged();
+        d->m_modelResetInProgress = false;
+        Q_EMIT sourceModelChanged();
     }
 }
 
-QHash< int, QByteArray > PaginateModel::roleNames() const
+QHash<int, QByteArray> PaginateModel::roleNames() const
 {
     return d->m_sourceModel ? d->m_sourceModel->roleNames() : QAbstractItemModel::roleNames();
 }
 
 int PaginateModel::rowsByPageSize(int size) const
 {
-    return d->m_hasStaticRowCount ? size
-        : !d->m_sourceModel ? 0
-        : qMin(d->m_sourceModel->rowCount() - d->m_firstItem, size);
+    return d->m_hasStaticRowCount ? size : !d->m_sourceModel ? 0 : qMin(d->m_sourceModel->rowCount() - d->m_firstItem, size);
 }
 
-int PaginateModel::rowCount(const QModelIndex& parent) const
+int PaginateModel::rowCount(const QModelIndex &parent) const
 {
     return parent.isValid() ? 0 : rowsByPageSize(d->m_pageSize);
 }
 
-QModelIndex PaginateModel::mapToSource(const QModelIndex& idx) const
+QModelIndex PaginateModel::mapToSource(const QModelIndex &idx) const
 {
     if (!d->m_sourceModel)
         return QModelIndex();
     return d->m_sourceModel->index(idx.row() + d->m_firstItem, idx.column());
 }
 
-QModelIndex PaginateModel::mapFromSource(const QModelIndex& idx) const
+QModelIndex PaginateModel::mapFromSource(const QModelIndex &idx) const
 {
     Q_ASSERT(idx.model() == d->m_sourceModel);
     if (!d->m_sourceModel)
@@ -156,7 +146,7 @@ QModelIndex PaginateModel::mapFromSource(const QModelIndex& idx) const
     return index(idx.row() - d->m_firstItem, idx.column());
 }
 
-QVariant PaginateModel::data(const QModelIndex& index, int role) const
+QVariant PaginateModel::data(const QModelIndex &index, int role) const
 {
     if (!d->m_sourceModel)
         return QVariant();
@@ -186,12 +176,15 @@ void PaginateModel::previousPage()
 
 int PaginateModel::currentPage() const
 {
+    if (d->m_pageSize == 0)
+        return 0;
+
     return d->m_firstItem / d->m_pageSize;
 }
 
 int PaginateModel::pageCount() const
 {
-    if (!d->m_sourceModel)
+    if (!d->m_sourceModel || d->m_pageSize == 0)
         return 0;
     const int rc = d->m_sourceModel->rowCount();
     const int r = (rc % d->m_pageSize == 0) ? 1 : 0;
@@ -209,53 +202,59 @@ void PaginateModel::setStaticRowCount(bool src)
         return;
     }
 
+    d->m_modelResetInProgress = true;
     beginResetModel();
     d->m_hasStaticRowCount = src;
     endResetModel();
+    d->m_modelResetInProgress = false;
 
     Q_EMIT staticRowCountChanged();
 }
 
 //////////////////////////////
 
-void PaginateModel::_k_sourceColumnsAboutToBeInserted(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceColumnsAboutToBeInserted(const QModelIndex &parent, int start, int end)
 {
     Q_UNUSED(end)
-    if (parent.isValid() || start!=0) {
+    if (parent.isValid() || start != 0) {
         return;
     }
+    d->m_modelResetInProgress = true;
     beginResetModel();
 }
 
-void PaginateModel::_k_sourceColumnsAboutToBeMoved(const QModelIndex& sourceParent, int sourceStart, int sourceEnd, const QModelIndex& destParent, int dest)
+void PaginateModel::_k_sourceColumnsAboutToBeMoved(const QModelIndex &sourceParent, int sourceStart, int sourceEnd, const QModelIndex &destParent, int dest)
 {
     Q_UNUSED(sourceParent)
     Q_UNUSED(sourceStart)
     Q_UNUSED(sourceEnd)
     Q_UNUSED(destParent)
     Q_UNUSED(dest)
+    d->m_modelResetInProgress = true;
     beginResetModel();
 }
 
-void PaginateModel::_k_sourceColumnsAboutToBeRemoved(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceColumnsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
 {
     Q_UNUSED(end)
-    if (parent.isValid() || start!=0) {
+    if (parent.isValid() || start != 0) {
         return;
     }
+    d->m_modelResetInProgress = true;
     beginResetModel();
 }
 
-void PaginateModel::_k_sourceColumnsInserted(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceColumnsInserted(const QModelIndex &parent, int start, int end)
 {
     Q_UNUSED(end)
-    if (parent.isValid() || start!=0) {
+    if (parent.isValid() || start != 0) {
         return;
     }
     endResetModel();
+    d->m_modelResetInProgress = false;
 }
 
-void PaginateModel::_k_sourceColumnsMoved(const QModelIndex& sourceParent, int sourceStart, int sourceEnd, const QModelIndex& destParent, int dest)
+void PaginateModel::_k_sourceColumnsMoved(const QModelIndex &sourceParent, int sourceStart, int sourceEnd, const QModelIndex &destParent, int dest)
 {
     Q_UNUSED(sourceParent)
     Q_UNUSED(sourceStart)
@@ -263,20 +262,22 @@ void PaginateModel::_k_sourceColumnsMoved(const QModelIndex& sourceParent, int s
     Q_UNUSED(destParent)
     Q_UNUSED(dest)
     endResetModel();
+    d->m_modelResetInProgress = false;
 }
 
-void PaginateModel::_k_sourceColumnsRemoved(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceColumnsRemoved(const QModelIndex &parent, int start, int end)
 {
     Q_UNUSED(end)
-    if (parent.isValid() || start!=0) {
+    if (parent.isValid() || start != 0) {
         return;
     }
     endResetModel();
+    d->m_modelResetInProgress = false;
 }
 
-void PaginateModel::_k_sourceDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight, const QVector<int> &roles)
+void PaginateModel::_k_sourceDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
 {
-    if (topLeft.parent().isValid() || bottomRight.row() < d->m_firstItem || topLeft.row()>lastItem()) {
+    if (topLeft.parent().isValid() || bottomRight.row() < d->m_firstItem || topLeft.row() > lastItem()) {
         return;
     }
 
@@ -285,39 +286,41 @@ void PaginateModel::_k_sourceDataChanged(const QModelIndex& topLeft, const QMode
     if (!idxTop.isValid())
         idxTop = index(0);
     if (!idxBottom.isValid())
-        idxBottom = index(rowCount()-1);
+        idxBottom = index(rowCount() - 1);
 
-    emit dataChanged(idxTop, idxBottom, roles);
+    Q_EMIT dataChanged(idxTop, idxBottom, roles);
 }
 
 void PaginateModel::_k_sourceHeaderDataChanged(Qt::Orientation orientation, int first, int last)
 {
     Q_UNUSED(last)
     if (first == 0)
-        emit headerDataChanged(orientation, 0, 0);
+        Q_EMIT headerDataChanged(orientation, 0, 0);
 }
 
 void PaginateModel::_k_sourceModelAboutToBeReset()
 {
+    d->m_modelResetInProgress = true;
     beginResetModel();
 }
 
 void PaginateModel::_k_sourceModelReset()
 {
     endResetModel();
+    d->m_modelResetInProgress = false;
 }
 
-bool PaginateModel::isIntervalValid(const QModelIndex& parent, int start, int /*end*/) const
+bool PaginateModel::isIntervalValid(const QModelIndex &parent, int start, int /*end*/) const
 {
-    return !parent.isValid() && start<=lastItem();
+    return !parent.isValid() && start <= lastItem();
 }
 
 bool PaginateModel::canSizeChange() const
 {
-    return !d->m_hasStaticRowCount && currentPage() == pageCount()-1;
+    return !d->m_hasStaticRowCount && currentPage() == pageCount() - 1;
 }
 
-void PaginateModel::_k_sourceRowsAboutToBeInserted(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceRowsAboutToBeInserted(const QModelIndex &parent, int start, int end)
 {
     if (!isIntervalValid(parent, start, end)) {
         return;
@@ -325,38 +328,41 @@ void PaginateModel::_k_sourceRowsAboutToBeInserted(const QModelIndex& parent, in
 
     if (canSizeChange()) {
         const int newStart = qMax(start - d->m_firstItem, 0);
-        const int insertedCount = qMin(end - start, pageSize() - newStart -1);
-        beginInsertRows(QModelIndex(), newStart, newStart+insertedCount);
+        const int insertedCount = qMin(end - start, pageSize() - newStart - 1);
+        beginInsertRows(QModelIndex(), newStart, newStart + insertedCount);
     } else {
+        d->m_modelResetInProgress = true;
         beginResetModel();
     }
 }
 
-void PaginateModel::_k_sourceRowsInserted(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceRowsInserted(const QModelIndex &parent, int start, int end)
 {
     if (!isIntervalValid(parent, start, end)) {
         return;
     }
 
-    if (canSizeChange()) {
-        endInsertRows();
-    } else {
+    if (d->m_modelResetInProgress) {
         endResetModel();
+        d->m_modelResetInProgress = false;
+    } else {
+        endInsertRows();
     }
 }
 
-void PaginateModel::_k_sourceRowsAboutToBeMoved(const QModelIndex& sourceParent, int sourceStart, int sourceEnd, const QModelIndex& destParent, int dest)
+void PaginateModel::_k_sourceRowsAboutToBeMoved(const QModelIndex &sourceParent, int sourceStart, int sourceEnd, const QModelIndex &destParent, int dest)
 {
     Q_UNUSED(sourceParent)
     Q_UNUSED(sourceStart)
     Q_UNUSED(sourceEnd)
     Q_UNUSED(destParent)
     Q_UNUSED(dest)
-    //NOTE could optimize, unsure if it makes sense
+    // NOTE could optimize, unsure if it makes sense
+    d->m_modelResetInProgress = true;
     beginResetModel();
 }
 
-void PaginateModel::_k_sourceRowsMoved(const QModelIndex& sourceParent, int sourceStart, int sourceEnd, const QModelIndex& destParent, int dest)
+void PaginateModel::_k_sourceRowsMoved(const QModelIndex &sourceParent, int sourceStart, int sourceEnd, const QModelIndex &destParent, int dest)
 {
     Q_UNUSED(sourceParent)
     Q_UNUSED(sourceStart)
@@ -364,24 +370,26 @@ void PaginateModel::_k_sourceRowsMoved(const QModelIndex& sourceParent, int sour
     Q_UNUSED(destParent)
     Q_UNUSED(dest)
     endResetModel();
+    d->m_modelResetInProgress = false;
 }
 
-void PaginateModel::_k_sourceRowsAboutToBeRemoved(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceRowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
 {
     if (!isIntervalValid(parent, start, end)) {
         return;
     }
 
     if (canSizeChange()) {
-        const int removedCount = end-start;
+        const int removedCount = end - start;
         const int newStart = qMax(start - d->m_firstItem, 0);
         beginRemoveRows(QModelIndex(), newStart, newStart + removedCount);
     } else {
+        d->m_modelResetInProgress = true;
         beginResetModel();
     }
 }
 
-void PaginateModel::_k_sourceRowsRemoved(const QModelIndex& parent, int start, int end)
+void PaginateModel::_k_sourceRowsRemoved(const QModelIndex &parent, int start, int end)
 {
     if (!isIntervalValid(parent, start, end)) {
         return;
@@ -390,6 +398,7 @@ void PaginateModel::_k_sourceRowsRemoved(const QModelIndex& parent, int start, i
     if (canSizeChange()) {
         endRemoveRows();
     } else {
+        d->m_modelResetInProgress = true;
         beginResetModel();
     }
 }
